# 特征Trait
如果我们想定义一个文件系统，那么把该系统跟底层存储解耦是很重要的。文件操作主要包含三个：`open`、`write`、`read`, 这些操作可以发生在硬盘，也可以发生在缓存，可以通过网络也可以通过(我实在编不下去了，大家来帮帮我)。总之如果你要为每一种情况都单独实现一套代码，那这种实现将过于繁杂，而且也没那个必要。

要解决上述把某种行为抽象出来的问题，就要使用Rust中的特征`trait`概念。可能你是第一次听说这个名词，但是不要怕，如果学过其他语言，那么大概率你听说过接口，没错，特征很类似接口。

在之前的代码中，我们也多次见过特征的使用，例如`#[derive(Debug)]`，它在自定义的类型上自动派生`Debug`特征，接着可以使用`println!("{:?}",x)`打印自定义的类型;再例如:
```rust
fn add<T: std::ops::Add<Output = T>>(a:T, b:T) -> T {
    a + b
}
```
通过`std::ops::Add`特征来限制`T`，这样才能进行合法的加法操作，否则不可能任何类型都能进行相加。

这些都说明一个道理，特征定义了**一个可以被共享的行为，只要实现了特征，你就能使用该行为**。

## 定义特征
如果不同的类型具有相同的行为，那么我们就可以定义一个特征，然后为这些类型实现该特征。**定义特征**是把一些方法组合在一起，目的是定义一个实现某些目标所必需的行为的集合。

例如，我们现在有文章`Post`和微博`Weibo`两种内容载体，而我们想对相应的内容进行总结，也就是无论是文章内容，还是微博内容，都可以在某个时间点进行总结，那么总结这个行为就是共享的，因此可以用特征来定义：
```rust
pub trait Summary {
    fn summarize(&self) -> String;
}
```

这里使用 `trait` 关键字来声明一个特征，`Summary`是特征名。在大括号中定义描述该特征的所有方法，在这个例子中是`fn summarize(&self) -> String`。

特征只定义行为看起来是什么样的，而不定义行为具体是怎么样的。因此这里，我们只定义特征方法的签名，而不进行实现，因此方法后面是`;`，而不是一个`{}`。

接着每一个实现这个特征的类型都需要具体实现该特征的相应方法，编译器也会确保任何实现`Summary`特征的类型都拥有与这个签名的定义完全一致的 `summarize` 方法。

## 为类型实现特征
因为特征只定义行为看起来是什么样的，因此我们需要为类型实现具体的特征，定义行为具体是怎么样的。

首先来为`Post`和`Weibo`实现`Summary`特征:
```rust
pub trait Summary {
    fn summarize(&self) -> String;
}
pub struct Post {
    pub title: String, // 标题
    pub author: String, // 作者
    pub content: String, // 内容
}

impl Summary for Post {
    fn summarize(&self) -> String {
        format!("文章{}, 作者是{}", self.title, self.author)
    }
}

pub struct Weibo {
    pub username: String,
    pub content: String
}

impl Summary for Weibo {
    fn summarize(&self) -> String {
        format!("{}发表了微博{}", self.username, self.content)
    }
}
```

实现特征的语法跟为结构体、枚举实现方法挺像: `impl Summary for Post`，读作为`Post`类型实现`Summary`特征，然后在`impl`的花括号中实现该特征的具体方法。

接下来就可以在类型上调用特征的方法：
```rust
fn main() {
    let post = Post{title: "Rust语言简介".to_string(),author: "Sunface".to_string(), content: "Rust棒极了!".to_string()};
    let weibo = Weibo{username: "sunface".to_string(),content: "好像微博没Tweet好用".to_string()};
    
    post.summarize();
    weibo.summarize();
}
```

运行输出:
```console
文章Rust语言简介, 作者是Sunface
sunface发表了微博好像微博没Tweet好用
```

说实话，如果特征仅仅如此，你可能会觉得花里胡哨没啥子用，等下就让你见识下真正的威力。

#### 特征定义与实现的位置(孤儿规则)
上面我们将`Summary`定义为了`pub`公开的，因此如果他人想要使用我们的`Summary`特征，则可以引入到他们的包中，然后再进行实现。

关于特征实现与定义的位置，有一条非常重要的原则: **如果你想要为类型`A`实现特征`T`，那么`A`或者`T`至少有一个是在当前作用域中定义的！**.例如我们可以为上面的`Post`类型实现标准库中的`Display`特征，这是因为`Post`类型定义在当前的作用域中。同时，我们也可以在当前包中为`String`类型实现`Summary`特征，因为`Summary`定义在当前作用域中。

但是你无法在当前作用域中，为`String`类型实现`Display`特征，因为它们两都定义在标准库中，跟你半毛钱关系都没有，看看就行了。

该规则被称为**孤儿规则**，可以确保其它人编写的代码不会破坏你的代码，也确保了你不会莫名其妙就破坏了风马牛不相及的代码。

#### 默认实现
你可以在特征中定义具有**默认实现**的方法，这样其它类型无需再实现该方法，或者也可以选择重载该方法:
```rust
pub trait Summary {
    fn summarize(&self) -> String {
        String::from("(Read more...)")
    }
}
```

上面为`Summary`定义了一个默认实现，下面我们编写段代码来测试下:
```rust
impl Summary for Post {}

impl Summary for Weibo {
    fn summarize(&self) -> String {
        format!("{}发表了微博{}", self.username, self.content)
    }
}
```

可以看到，`Post`选择了默认实现，而`Weibo`重载了该方法，调用和输出如下:
```rust
    println!("{}",post.summarize());
    println!("{}",weibo.summarize());
```

```console
(Read more...)
sunface发表了微博好像微博没Tweet好用
```

默认实现允许调用相同特种中的其他方法，哪怕这些方法没有默认实现。如此，特征可以提供很多有用的功能而只需要实现指定的一小部分内容。例如，我们可以定义`Summary`特征，使其具有一个需要实现的`summarize_author`方法，然后定义一个`summarize`方法，此方法的默认实现调用`summarize_author`方法：
```rust
pub trait Summary {
    fn summarize_author(&self) -> String;

    fn summarize(&self) -> String {
        format!("(Read more from {}...)", self.summarize_author())
    }
}
```

为了使用`Summary`，只需要实现`summarize_author`方法即可:
```rust
impl Summary for Weibo {
    fn summarize_author(&self) -> String {
        format!("@{}", self.username)
    }
}
println!("1 new weibo: {}", weibo.summarize());

```

`weibo.summarize()`会先调用`Summary`特征默认实现的`summarize`方法，通过该方法进而调用`Weibo`为`Summary`实现的`summarize_author`方法，最终输出：`1 new weibo: (Read more from @horse_ebooks...)`.

## 使用特征作为函数参数
之前提到过，特征如果仅仅是用来调用方法，那真的有些大材小用，现在我们来讲下，真正可以让特征大放光彩的地方。

现在，先定义一个函数，使用特征用做函数参数：
```rust
pub fn notify(item: &impl Summary) {
    println!("Breaking news! {}", item.summarize());
}
```

`impl Summary`，只能说出这个类型的人真的是起名鬼才，简直太贴切了，故名思义`实现了Summary特征`的`item参数.

你可以使用任何实现了`Summary`特征的类型作为该函数的参数，同时在函数体内，还可以调用该特征的的方法，例如`summarize`方法。具体的说，可以传递`Post`或`Weibo`的实例来作为参数，而其它类如`String`或者`i32`的类型则不能用做该函数的参数，因为它们没有实现`Summary`特征。

## 特征约束(trait bound)
虽然`impl Trait`这种语法非常好理解，但是实际上它只是一个语法糖:
```rust
pub fn notify<T: Summary>(item: &T) {
    println!("Breaking news! {}", item.summarize());
}
```
真正的完整形式如上所述，形如`T:Summary`被称为**特征约束**。

在简单的场景下`impl Trait`的语法就足够使用，但是对于复杂的场景，特征约束可以让我们拥有更大的灵活性和语法表现能力，例如一个函数接受两个`impl Summary`的参数：
```rust
pub fn notify(item1: &impl Summary, item2: &impl Summary) {}
```

如果函数两个参数是不同的类型，那么上面的方法很好，只要这两个类型都实现了`Summary`特征即可。但是如果我们想要强制函数的两个参数是同一类型呢？上面的语法就无法做到这种限制，此时我们只能使特征约束来实现：
```rust
pub fn notify<T: Summary>(item1: &T, item2: &T) {}
```

泛型类型`T`说明了`item1`和`item2`必须拥有同样的类型，同时`T: Summary`说明了`T`必须实现`Summary`特征。

#### 多重约束
除了单个约束条件，我们还可以指定多个约束条件，例如除了让参数实现`Summary`特征外，还可以让参数实现`Display`特征以控制它的格式化输出：
```rust
pub fn notify(item: &(impl Summary + Display)) {
```

除了上述的语法糖形式，还能使用特征约束的形式:
```rust
pub fn notify<T: Summary + Display>(item: &T) {}
```
通过这两个特征，就可以使用`item.summarize`方法，以及通过`println!("{}",item)`来格式化输出`item`。

#### Where约束
当特征约束变得很多时，函数的签名将变得很复杂：
```rust
fn some_function<T: Display + Clone, U: Clone + Debug>(t: &T, u: &U) -> i32 {
```
严格来说，上面的例子还是不够复杂，但是我们还是能对其做一些形式上的改进，通过`where`：
```rust
fn some_function<T, U>(t: &T, u: &U) -> i32
    where T: Display + Clone,
          U: Clone + Debug
{
```

#### 使用特征约束有条件的实现方法或特征
特征约束，可以让我们在指定类型 + 指定特征的条件下去实现方法，例如:
```rust
use std::fmt::Display;

struct Pair<T> {
    x: T,
    y: T,
}

impl<T> Pair<T> {
    fn new(x: T, y: T) -> Self {
        Self {
            x,
            y,
        }
    }
}

impl<T: Display + PartialOrd> Pair<T> {
    fn cmp_display(&self) {
        if self.x >= self.y {
            println!("The largest member is x = {}", self.x);
        } else {
            println!("The largest member is y = {}", self.y);
        }
    }
}
```

`cmd_display`方法，并不是所有的`Pair<T>`结构体对象都拥有，只有`T`实现了`Display + PartialOrd`的`Part<T>`才拥有此方法
该函数可读性会更好，因为泛型参数、参数、返回值都在一起，可以快速的阅读，同时每个泛型参数的特征也在新的代码行中通过`where`进行了约束。

**也可以有条件的实现特征**, 例如，标准库为任何实现了 `Display`特征的类型实现了 `ToString`特征：
```rust
impl<T: Display> ToString for T {
    // --snip--
}
```

我们可以对任何实现了 `Display` 特征的类型调用由 `ToString` 定义的 `to_string` 方法。例如，可以将整型转换为对应的 `String` 值，因为整型实现了 `Display`：
```rust
let s = 3.to_string();
```


## 函数返回中的`impl Trait`
可以通过`impl Trait`来说明一个函数返回了一个类型，该类型实现了某个特征:
```rust
fn returns_summarizable() -> impl Summary {
    Weibo {
        username: String::from("sunface"),
        content: String::from(
            "m1 max太厉害了，电脑再也不会卡",
        )
    }
}
```

因为`Weibo`实现了`Summary`，因此这里可以用它来作为返回值。要注意的是，虽然我们知道这里是一个`Weibo`类型，但是对于`returns_summarizable`的调用者而言，他只知道返回了一个实现了`Summary`特征的对象，但是并不知道返回了一个`Weibo`类型.

这种`impl Trait`形式的返回值，在一种场景下非常非常有用，那就是返回的真实类型非常复杂，你不知道该怎么声明时(毕竟Rust要求你必须标出所有的类型)，此时就可以用`impl Trait`的方式简单返回。例如，闭包和迭代器就是很复杂，只有编译器才知道那玩意的真实类型，如果让你写出来它们的具体类型，我估计想杀人的心都有，好在你可以用`impl Iterator`来告诉调用者，返回了一个迭代器，因为所有迭代器都会实现`Iterator`特征。

但是这种返回值方式有一个很大的限制：只能有一个具体的类型，例如：
```rust
fn returns_summarizable(switch: bool) -> impl Summary {
    if switch {
        Post {
            title: String::from(
                "Penguins win the Stanley Cup Championship!",
            ),
            author: String::from("Iceburgh"),
            content: String::from(
                "The Pittsburgh Penguins once again are the best \
                 hockey team in the NHL.",
            ),
        }
    } else {
        Weibo {
            username: String::from("horse_ebooks"),
            content: String::from(
                "of course, as you probably already know, people",
            ),
        }
    }
}
```

以上的代码就无法通过编译，因为它返回了两个不同的类型，

```console
`if` and `else` have incompatible types
expected struct `Post`, found struct `Weibo`
```

报错提示我们`if`和`else`返回了不同的类型。如果想要实现返回不同的类型，需要使用下一章节中的[特征对象](./trait-object.md).

## 修复上一节中的`largest`函数
还记得上一节中的[例子](./generic#泛型详解)吧,当时留下一个疑问，该如何解决编译报错：
```rust
error[E0369]: binary operation `>` cannot be applied to type `T` // 无法在`T`类型上应用`>`运算符
 --> src/main.rs:5:17
  |
5 |         if item > largest {
  |            ---- ^ ------- T
  |            |
  |            T
  |
help: consider restricting type parameter `T` // 考虑使用以下的特征来约束T
  |
1 | fn largest<T: std::cmp::PartialOrd>(list: &[T]) -> T {
  |             ^^^^^^^^^^^^^^^^^^^^^^
```

在 `largest` 函数体中我们想要使用大于运算符（>）比较两个 `T` 类型的值。这个运算符是标准库中特征 `std::cmp::PartialOrd` 的一个默认方法。所以需要在 `T` 的特征约束中指定 `PartialOrd`，这样 `largest` 函数可以用于内部元素类型可以比较大小的数组切片.

由于`PartialOrd` 位于 `prelude` 中所以并不需要通过`std::cmp`手动将其引入作用域。将 `largest` 的签名修改为如下：
```rust
fn largest<T: PartialOrd>(list: &[T]) -> T {
```

但是此时编译，又会出现新的错误：
```rust
error[E0508]: cannot move out of type `[T]`, a non-copy slice
 --> src/main.rs:2:23
  |
2 |     let mut largest = list[0];
  |                       ^^^^^^^
  |                       |
  |                       cannot move out of here
  |                       help: consider using a reference instead: `&list[0]`

error[E0507]: cannot move out of borrowed content
 --> src/main.rs:4:9
  |
4 |     for &item in list.iter() {
  |         ^----
  |         ||
  |         |hint: to prevent move, use `ref item` or `ref mut item`
  |         cannot move out of borrowed content
```

错误的核心是 `cannot move out of type [T], a non-copy slice`， 原因是`T`没有[实现`Copy`特性](../ownership/ownership.md#拷贝(浅拷贝))，因此我们只能把所有权进行转移，毕竟只有`i32`等基础类型才实现了`Copy`特性，可以存储在栈上，而`T`可以指代任何类型(严格来说是实现了`PartialOrd`特征的所有类型)。

因此，为了让T拥有`Copy`特性，我们可以加上特征约束:
```rust
fn largest<T: PartialOrd + Copy>(list: &[T]) -> T {
    let mut largest = list[0];

    for &item in list.iter() {
        if item > largest {
            largest = item;
        }
    }

    largest
}

fn main() {
    let number_list = vec![34, 50, 25, 100, 65];

    let result = largest(&number_list);
    println!("The largest number is {}", result);

    let char_list = vec!['y', 'm', 'a', 'q'];

    let result = largest(&char_list);
    println!("The largest char is {}", result);
}
```

如果并不希望限制 `largest` 函数只能用于实现了 `Copy` 特征的类型，我们可以在 `T` 的特征约束中指定 [`Clone`特征](../ownership/ownership.md#克隆(深拷贝)) 而不是 `Copy`特征。并克隆`list`中的每一个值使得 `largest` 函数拥有其所有权。使用 `clone` 函数意味着对于类似 String 这样拥有堆上数据的类型，会潜在的分配更多堆上空间，而堆分配在涉及大量数据时可能会相当缓慢。

另一种 `largest` 的实现方式是返回在 `list` 中 `T` 值的引用。如果我们将函数返回值从 `T` 改为 `&T` 并改变函数体使其能够返回一个引用，我们将不需要任何 `Clone` 或 `Copy` 的特征约束而且也不会有任何的堆分配。尝试自己实现这种替代解决方式吧！


## 通过`derive`派生特征
在本书中，形如`#[derive(Debug)]`的代码已经出现了很多次，这种是一种特征派生语法，被`derive`标记的对象会自动实现对应的默认特征代码，继承相应的功能。

例如`Debug`特征，它有一套自动实现的默认代码，当你给一个结构体标记后，就可以使用`println!("{:?}",s)`的形式打印该结构体的对象。

例如`Copy`特征，它也有一套自动实现的默认代码，当标记到一个类型上时，可以让这个类型自动实现`Copy`特征，进而可以调用`copy`方法，进行自我复制。

总是，`derive`派生出来的是Rust默认给我们提供的特征，在开发过程中极大的简化了自己手动实现相应特征的需求，当然，如果你有特殊的需求，还可以自己手动重载该实现。

详细的`derive`列表参加[附录-派生特征](../../appendix/derive.md).

## 调用方法需要引入特征
在一些场景中，使用`as`关键字做类型转换会有比较大的限制，因为你想要在类型转换上拥有完全的控制，例如处理转换错误，那么你将需要`TryInto`:

```rust
use std::convert::TryInto;
 
fn main() {
  let a: i32 = 10;
  let b: u16 = 100;

  let b_ = b.try_into()
            .unwrap();

  if a < b_ {
    println!("Ten is less than one hundred.");
  }
}
```

上面代码中引入了`std::convert::TryInto`特征，但是却没有使用它，可能有些同学会为此困惑，主要原因在于**如果你要使用一个特征的方法，那么你需要引入该特征到当前的作用域中**，我们在上面用到了`try_into`方法，因此需要引入对应的特征。

但是Rust又提供了一个非常便利的办法，即把最常用的标准库中的特征通过[`std::prelude`](std::convert::TryInto)模块提前引入到当前作用域中,其中包括了`std::convert::TryInto`，你可以尝试删除第一行的代码`use ...`，看看是否会报错.


## 几个综合例子

#### 为自定义类型实现`+`操作
在Rust中除了数值类型的加法，`String`也可以做[加法](../compound-type/string-slice.md#操作字符串)，因为Rust为该类型实现了`std::ops::Add`特征，同理，如果我们为自定义类型实现了该特征，那就可以实现`Point1 + Point2`的操作: 

```rust
use std::ops::Add;

// 为Point结构体派生Debug特征，用于格式化输出
#[derive(Debug)]
struct Point<T: Add<T, Output = T>> { //限制类型T必须实现了Add特征，否则无法进行+操作。
    x: T,
    y: T,
}

impl<T: Add<T, Output = T>> Add for Point<T> {
    type Output = Point<T>;

    fn add(self, p: Point<T>) -> Point<T> {
        Point{
            x: self.x + p.x,
            y: self.y + p.y,
        }
    }
}

fn add<T: Add<T, Output=T>>(a:T, b:T) -> T {
    a + b
}

fn main() {
    let p1 = Point{x: 1.1f32, y: 1.1f32};
    let p2 = Point{x: 2.1f32, y: 2.1f32};
    println!("{:?}", add(p1, p2));

    let p3 = Point{x: 1i32, y: 1i32};
    let p4 = Point{x: 2i32, y: 2i32};
    println!("{:?}", add(p3, p4));
}
```

#### 自定义类型的打印输出
在开发过程中，往往只要使用`#[derive(Debug)]`对我们的自定义类型进行标注，即可实现打印输出的功能：
```rust
#[derive(Debug)]
struct Point{
    x: i32,
    y: i32
}
fn main() {
    let p = Point{x:3,y:3};
    println!("{:?}",p);
}
```
但是在实际项目中，往往需要对我们的自定义类型进行自定义的格式化输出，以让用户更好的阅读理解我们的类型，此时就要为自定义类型实现`std::fmt::Display`特征：
```rust
#![allow(dead_code)]
 
use std::fmt;
use std::fmt::{Display};
 
#[derive(Debug,PartialEq)]
enum FileState {
  Open,
  Closed,
}
 
#[derive(Debug)]
struct File {
  name: String,
  data: Vec<u8>,
  state: FileState,
}
 
impl Display for FileState {
   fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
     match *self {
         FileState::Open => write!(f, "OPEN"),
         FileState::Closed => write!(f, "CLOSED"),
     }
   }
}
 
impl Display for File {
   fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
      write!(f, "<{} ({})>",
             self.name, self.state)
   }
}
 
impl File {
  fn new(name: &str) -> File {
    File {
        name: String::from(name),
        data: Vec::new(),
        state: FileState::Closed,
    }
  }
}
 
fn main() {
  let f6 = File::new("f6.txt");
  //...
  println!("{:?}", f6);
  println!("{}", f6);
}
```


以上两个例子较为复杂，目的是为读者展示下真实的使用场景长什么样，因此需要读者细细阅读，最终消化这些知识对于你的Rust之路会有莫大的帮助。


最后，特征和特征约束，是Rust中极其重要的概念，如果你还是没搞懂，强烈建议回头再看一遍，或者寻找相关的资料进行补充学习。如果已经觉得掌握了，那么就进入下一节的学习。